/****************************************************************************\
**  Title:       PCXENCOD.C                                                 **
**  Purpose:     Encode a PCX image file scan line.                         **
**  Version:     1.0                                                        **
**  Date:        May 1991                                                   **
**  Author:      James D. Murray, Anaheim, CA  USA                          **
**  C Compilers: Borland C++ v2.0, Microsoft C v6.00a                       **
**                                                                          **
**  Compress an image scan line using the PCX run-length encoding           **
**  algorithm.  Useful for algorithms that need to work on images one scan  **
**  line at a time.                                                         **
**                                                                          **
**  This module contains the following functions:                           **
**                                                                          **
**      PcxEncodeScanLine - Encode a raw scan line using PCX RLE.           **
**                                                                          **
**  Copyright (C) 1991 by Graphics Software Labs.  All rights reserved.     **
\****************************************************************************/
#include <stdio.h>
#include "endianio.h"
#include "pcx.h"

/*
**  Encode (compress) a PCX image file scan line.
**
**  PcxEncodeScanLine() encodes a buffer containing raw scan line data
**  using the PCX PackBytes run-length encoding algorithm.  The encoded
**  data is written to a second buffer passed to the function.
**
**  The main problem when encoding a scan line is that we never know how
**  long the resulting encoded scan line will be.  Typically it will be
**  shorter than the unencoded line, however, this type of RLE encoding
**  can make the resulting encoded data larger in size than the unencoded
**  data depending upon the data being encoded.
**
**  It is therefore important that the buffer used to hold the encoded
**  scan line be larger than what will typically be needed.  A size of
**  2048 bytes should be more than plenty for typical VGA images.  The
**  size of the buffer can be trimmed down to the size of the encoded
**  data using realloc() and the return value of PcxEncodeScanLine()
**  function, if necessary.
**
**  Returns: Number of bytes in compressed scan line.
*/
WORD
PcxEncodeScanLine(DecodedBuffer, EncodedBuffer, BufferSize)
BYTE *DecodedBuffer;    /* Pointer to buffer holding unencoded data         */
BYTE *EncodedBuffer;    /* Pointer to buffer to hold encodeded scan line    */
WORD  BufferSize;       /* Size of buffer holding unencoded data            */
{
    register WORD index = 0;        /* Index into uncompressed data buffer  */
    register WORD scanindex = 0;    /* Index into compressed data buffer    */
    BYTE runcount;                  /* Length of encoded pixel run          */
    BYTE runvalue;                  /* Value of encoded pixel run           */

    while (index < BufferSize)
    {
        /*
        ** Get the run count of the next pixel value run.
        **
        ** Pixel value runs are encoded until a different pixel value
        ** is encountered, the end of the scan line is reached, or 63
        ** pixel values have been counted.
        */
        for (runcount = 1, runvalue = DecodedBuffer[index];
             runvalue == DecodedBuffer[index + runcount] &&
             index + runcount < BufferSize &&
             runcount < 63;
             runcount++);

        /*
        ** Encode the run into a one or two-byte code.
        **
        ** Multiple pixel runs are stored in two-byte codes.  If a single
        ** pixel run has a value of less than 64 then it is stored in a
        ** one-byte code.  If a single pixel run has a value of 64 to 255
        ** then it is stored in a two-byte code.
        */
        if (runcount > 1)                   /* Multiple pixel run */
        {
            EncodedBuffer[scanindex++] = runcount | 0xC0;
            EncodedBuffer[scanindex++] = runvalue;       
        }
        else                                /* Single pixel run   */
        {
            if (DecodedBuffer[index] < 64)  /* Value is 0 to 63   */
            {
                EncodedBuffer[scanindex++] = runvalue;
            }
            else                            /* Value is 64 to 255 */
            {
                EncodedBuffer[scanindex++] = runcount | 0xC0;
                EncodedBuffer[scanindex++] = runvalue;       
            }
        }
        index += runcount;  /* Jump ahead to next pixel run value */
    }
    return(scanindex);      /* Return the number of bytes written to buffer */
}
}
